/*	$Csoft: button.c,v 1.95 2005/10/01 14:15:38 vedge Exp $	*/

/*
 * Copyright (c) 2005-2006 CubeSoft Communications, Inc.
 * <http://www.csoft.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <core/core.h>
#include <core/view.h>

#include <stdarg.h>
#include <string.h>

#include "button.h"

#include <gui/primitive.h>
#include <gui/text.h>
#include <gui/textview.h>

const AG_WidgetOps agTextviewOps = {
	{
		"AG_Widget:AG_Textview",
		sizeof(AG_Textview),
		{ 0,0 },
		NULL,		/* init */
		NULL,		/* reinit */
		AG_TextviewDestroy,
		NULL,		/* load */
		NULL,		/* save */
		NULL		/* edit */
	},
	AG_TextviewDraw,
	AG_TextviewScale
};

static void MouseButtonUp(AG_Event *);
static void MouseButtonDown(AG_Event *);
static void KeyUp(AG_Event *);
static void KeyDown(AG_Event *);

AG_Textview *
AG_TextviewNew(void *parent, Uint flags)
{
	AG_Textview *tx;

	tx = Malloc(sizeof(AG_Textview), M_OBJECT);
	AG_TextviewInit(tx, flags);
	AG_ObjectAttach(parent, tx);
	if (flags & AG_TEXTVIEW_FOCUS) {
		AG_WidgetFocus(tx);
	}
	return (tx);
}

void
AG_TextviewInit(AG_Textview *tx, Uint flags)
{
	Uint wflags = AG_WIDGET_FOCUSABLE|AG_WIDGET_CLIPPING;

	if (flags & AG_TEXTVIEW_HFILL) { wflags |= AG_WIDGET_HFILL; }
	if (flags & AG_TEXTVIEW_VFILL) { wflags |= AG_WIDGET_VFILL; }

	AG_WidgetInit(tx, "textview", &agTextviewOps, wflags);
	tx->flags = flags;
	tx->padding = 4;
	tx->lineskip = agTextFontLineSkip - agTextFontHeight;
	tx->lines = NULL;
	tx->m = 0;
	tx->i = 0;
	tx->j = 0;
	tx->mOffs = 0;
	tx->cBg = SDL_MapRGB(agVideoFmt, 0, 0, 0);
	tx->vBar = AG_ScrollbarNew(tx, AG_SCROLLBAR_VERT, 0);
	AG_WidgetBindInt(tx->vBar, "value", &tx->mOffs);
	AG_WidgetBindInt(tx->vBar, "max", &tx->nlines);

	AG_SetEvent(tx, "window-mousebuttondown", MouseButtonDown, NULL);
	AG_SetEvent(tx, "window-keyup", KeyUp, NULL);
	AG_SetEvent(tx, "window-keydown", KeyDown, NULL);
}

void
AG_TextviewScale(void *p, int w, int h)
{
	AG_Textview *tx = p;

	if (w == -1 && h == -1) {
		AGWIDGET(tx)->w = 1;
		AGWIDGET(tx)->h = 1;
		return;
	}
	AGWIDGET(tx->vBar)->x = AGWIDGET(tx)->w - 20;
	AGWIDGET(tx->vBar)->y = 0;
	AGWIDGET(tx->vBar)->w = 20;
	AGWIDGET(tx->vBar)->h = AGWIDGET(tx)->h;
	tx->vBar->visible = AGWIDGET(tx)->h / (agTextFontHeight+tx->lineskip);
}

void
AG_TextviewDraw(void *p)
{
	AG_Textview *tx = p;
	Uint r;
	int y;
	
	if (AGWIDGET(tx)->w < 8 || AGWIDGET(tx)->h < 8) { return; }
	if (tx->mOffs >= tx->m) { tx->mOffs = tx->m-1; }

	agPrim.box(tx, 0, 0, AGWIDGET(tx)->w, AGWIDGET(tx)->h, -1, tx->cBg);

	if (tx->m == 0)
		return;

	for (r = tx->mOffs, y = tx->padding;
	     r < tx->m && y < AGWIDGET(tx)->h;
	     r++) {
		AG_TextLine *ln = &tx->lines[r];

		
		
		y += su->h + tx->lineskip;
	}
}

void
AG_TextviewFreeLine(AG_TextLine *ln)
{
	Free(ln->s, M_WIDGET);
	Free(ln->cFg, M_WIDGET);
	Free(ln->cBg, M_WIDGET);
	Free(ln->flags, M_WIDGET);
}

void
AG_TextviewDestroy(void *p)
{
	AG_Textview *tx = p;
	int i;

	for (i = 0; i < tx->m; i++) {
		AG_TextviewFreeLine(&tx->lines[i]);
	}
	Free(tx->lines, M_WIDGET);
	tx->lines = NULL;
	AG_WidgetDestroy(tx);
}

static void
MouseButtonDown(AG_Event *event)
{
	AG_Textview *bu = AG_SELF();
	int button = AG_INT(1);
	
}

static void
KeyDown(AG_Event *event)
{
	AG_Textview *bu = AG_SELF();
	int keysym = AG_INT(1);
}

static void
KeyUp(AG_Event *event)
{
	AG_Textview *bu = AG_SELF();
	int keysym = AG_INT(1);
}

void
AG_TextviewSetPadding(AG_Textview *bu, int padding)
{
	bu->padding = padding;
}

AG_TextLine *
AG_TextviewAddLine(AG_Textview *tx, const char *s)
{
	AG_TextLine *ln;

	tx->lines = Realloc(tx->lines, (tx->m+1)*sizeof(AG_TextLine));
	ln = &tx->lines[tx->m++];
	ln->s = NULL;
	ln->n = 0;
	return (ln);
}

AG_TextChar *
AG_TextviewAddChar(AG_Textview *tx, AG_TextLine *ln, Uint32 c)
{
	AG_TextChar *tc;

	ln->s = Realloc(ln->s, (ln->n+1)*sizeof(AG_TextChar));
	tc = &ln->s[ln->n++];
	tc->c = c;
	tc->color = 0;
	tc->flags = 0;
	return (tc);
}

